/* Constructor functions for AST data types. */

#include "asttypes.h"
#include "astcons.h"
#include <assert.h>
#include <stdlib.h>

StmtList cons(Stmt s, StmtList l)
{
	StmtList tmp;

	tmp = malloc(sizeof(struct StmtList));
	assert(tmp != NULL);
	tmp->car = s;
	tmp->cdr = l;
	return tmp;
}

Stmt make_statement(void)
{
	Stmt tmp;

	tmp = calloc(1, sizeof(struct Stmt));
	assert(tmp != NULL);
	tmp->type = ST_INVALID;
	return tmp;
}

Stmt make_sbn_statement(Address lhs, Address rhs, Address branchdest)
{
	Stmt tmp = make_statement(); 

	tmp->type = ST_SBN;
	tmp->lhs = lhs;
	tmp->rhs = rhs;
	tmp->branchdest = branchdest;
	return tmp;
}

Stmt make_var_decl(int initial_value)
{
	Stmt tmp = make_statement();

	tmp->type = ST_VARDECL;
	tmp->value = initial_value;
	return tmp;
}

Address make_address(void)
{
	Address tmp;

	tmp = calloc(1, sizeof(struct Address));
	assert(tmp != NULL);
	tmp->type = AT_INVALID;
	return tmp;
}

Address make_pcrel(int offset, int pc)
{
	Address tmp = make_address();

	tmp->type = AT_PCREL;
	tmp->offset = offset;
	tmp->label = pc;
	return tmp;
}

Address make_lblrel(int offset, int label)
{
	Address tmp = make_address();

	tmp->type = AT_LABELREL;
	tmp->offset = offset;
	tmp->label = label;
	return tmp;
}

Address make_absolute(int addr)
{
	Address tmp = make_address();

	tmp->type = AT_ABSOLUTE;
	tmp->absolute = addr;
	return tmp;
}

