/* Symbol interning. */

#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#define HTSIZE 256

#define RHTSIZE 256

struct htent {
    char *key;
    int val;
    struct htent *next;
};

struct htent *table[HTSIZE];

struct rhtent {
    int key;
    char *val;
    struct rhtent *next;
};

struct rhtent *rtable[RHTSIZE];

int next_intern_num;

int hash(char *str)
{
	int acc = 0;

	while (*str++) { acc += *str; }
	return acc % HTSIZE;
}

int find(struct htent **t, char *key)
{
	struct htent *l;
	int hval = hash(key);

	l = t[hval];
    if (l == NULL) return -1;
    while (strcmp(key, l->key) != 0) {
		l = l->next;
		if (l == NULL) {
			return -1;
		}
	}
	return l->val;
}

int rhash(int key)
{
	return key % RHTSIZE;
}

char *rfind(struct rhtent **t, int key)
{
	struct rhtent *l;
	int hval = rhash(key);

	l = t[hval];
    if (l == NULL) return NULL;
    while (key != l->key) {
		l = l->next;
		if (l == NULL) {
			return NULL;
		}
	}
	return l->val;
}

void init_table(struct htent **t, int size)
{
	int x;

	for (x = 0; x < size; x++) {
		t[x] = NULL;
	}
}

void init_rtable(struct rhtent **t, int size)
{
	int x;

	for (x = 0; x < size; x++) {
		t[x] = NULL;
	}
}

void init_intern(void)
{
	next_intern_num = 1;
	init_table(table, HTSIZE);
	init_rtable(rtable, RHTSIZE);
}

void insert(struct htent **t, char *key, int value)
{
	struct htent *l, *ne, **dest;
	int hval = hash(key);

	l = t[hval];
    if (l == NULL) {
		dest = &t[hval];
	} else {
		dest = &(t[hval]->next);
	}
	ne = (struct htent *) malloc(sizeof(struct htent));
    assert(ne != NULL);
	ne->next = *dest;
    *dest = ne;
    ne->val = value;
	ne->key = strdup(key);
}

void rinsert(struct rhtent **t, int key, char *value)
{
	struct rhtent *l, *ne, **dest;
	int hval = rhash(key);

	l = t[hval];
    if (l == NULL) {
		dest = &t[hval];
	} else {
		dest = &(t[hval]->next);
	}
	ne = (struct rhtent *) malloc(sizeof(struct rhtent));
    assert(ne != NULL);
	ne->next = *dest;
    *dest = ne;
    ne->val = strdup(value);
	ne->key = key;
}

int intern(char *outside)
{
    int inside;

    inside = find(table, outside);
    if (inside != -1) return inside;

    insert(table, outside, next_intern_num);
    rinsert(rtable, next_intern_num, outside);
	next_intern_num += 1;
    return find(table, outside);
}

char *string_of(int intern_num)
{
	return rfind(rtable, intern_num);
}

int intern_tolower(char *outside)
{
	char buff[2000];
	int x;

	strncpy(buff, outside, 2000);
	for (x = 0; x < strlen(buff); x++) {
		if (isupper(buff[x])) {
			buff[x] = tolower(buff[x]);
		}
	}
	return intern(buff);
}

