/* Driver for SBN interpreter. */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "asttypes.h"

extern int yyparse();

StmtList the_program;
char *the_filename;
int quiet = 0;
int dump_before = 0;
char *output_only = NULL;

extern void simulate(StmtList prog);
extern void init_intern(void);
extern void dump_decls(void);

void help(char *myname)
{
	fprintf(stderr,"usage: %s [-hqb] [-o output_symbol] <program-file-name>\n",
	 myname);
	fprintf(stderr,"\tsimulator for the Single Instruction Computer\n"
	 "\twritten in a fit of madness by Brian R. Gaeke\n"
	 "\t\t<brg@CSUA.Berkeley.EDU>\n\n"
	 "\toption\tmeaning\n"
	 "\t-h\tprint usage summary\n"
	 "\t-q\tquiet! i.e., don't want verbose instruction tracing\n"
	 "\t-b\tdump variables before as well as after running program\n"
	 "\t-o SYM\toutput only the value of symbol SYM at end of run\n\n"
	 "\tProgram file format:   A program file for this simulator consists\n"
	 "\tof a series of one or more optionally labeled VARs or SBNs,\n"
	 "\twhere VARs are declarations of the form  .WORD N  (N is some\n"
	 "\tdecimal integer), and SBNs are statements of the form   SBN D, S, B\n"
     "\tmeaning basically  D -= S; if (D < 0) goto B;  D, S, and B\n"
	 "\tare addresses, which may be specified as a zero-based statement\n"
	 "\tnumber counting from the beginning of the file, or a label or\n"
	 "\t. (the current instruction) plus or minus a decimal offset.\n"
	 "\tFor example, .+1, LOOP-5, EXIT, and 37 are all valid addresses\n"
	 "\t(provided LOOP and EXIT are defined somewhere in your program.)\n"
     "\tProgram execution starts at a label START, which must be defined in\n"
     "\tyour program.  Bug reports, flames, etc., to /dev/null, please.\n");
	exit(1);
}

void usage_err(char *myname, char *errmsg)
{
	fprintf(stderr,"ERROR: %s\n",errmsg);
	help(myname);
}

int main(int argc, char **argv)
{
	extern FILE *yyin;
	int arg;

	while ((arg = getopt(argc, argv, "hqbo:")) != -1) {
		switch(arg) {
			case 'h': help(argv[0]); break;
			case 'b': dump_before++; break;
			case 'q': quiet++; break;
			case 'o': output_only = strdup(optarg); break;
			default: usage_err(argv[0], "invalid option specified"); break;
		}
	}
	if (argc - optind == 1) {
		the_filename = argv[optind];
	} else {
		usage_err(argv[0],
			"no program file name specified, or junk at end of args");
	}

	yyin = fopen(the_filename, "r");
	if (!yyin) {
		perror(the_filename);
		exit(1);
	}
	init_intern();
	if (yyparse() != 0) {
		fprintf(stderr, "%s: program parsing failed, aborting\n", argv[0]);
		exit(1);
	}
	simulate(the_program);
	dump_decls();
	exit(0);
}

