/* Parser for SBN interpreter. */

%{

#define YYERROR_VERBOSE

#include "asttypes.h"
#include "astcons.h"
#include <assert.h>
#include <stdlib.h>
#include <string.h>

extern StmtList the_program;
extern char *the_filename;

extern int yylex(void);

static int curr_statement_no = 0;

int yyerror(char *msg);

%}

%token ':' ',' '.' '-' '+'
%token SBN DOT_WORD
%token<I> ID DEC_CONST

%union {
	StmtList SL;
    Stmt S;
    Address A;
	int I;
}

%type<SL> Program
%type<S> Statement BareStatement SbnStatement VarDecl
%type<A> Addr
%type<I> Number

%%

Start: Program    { the_program = $1; }
     ;

Program: Statement         { $$ = cons($1, NULL); }
       | Statement Program { $$ = cons($1, $2); }
       ;

Statement: ID ':' BareStatement { $$ = $3;
                                  $$->addr = curr_statement_no++;
                                  $$->label = $1; }
         | BareStatement        { $$ = $1;
                                  $$->addr = curr_statement_no++; }
         ;

BareStatement: SbnStatement
             | VarDecl
             ;

SbnStatement: SBN Addr ',' Addr ',' Addr
                { $$ = make_sbn_statement($2, $4, $6); }
            ;

VarDecl: DOT_WORD Number { $$ = make_var_decl($2); }
       ;

Addr: '.'            { $$ = make_pcrel(0, curr_statement_no); }
    | '.' '+' Number { $$ = make_pcrel($3, curr_statement_no); }
    | '.' '-' Number { $$ = make_pcrel(-($3), curr_statement_no); }
    | ID             { $$ = make_lblrel(0, $1); }
    | ID '+' Number  { $$ = make_lblrel($3, $1); }
    | ID '-' Number  { $$ = make_lblrel(-($3), $1); }
    | Number         { $$ = make_absolute($1); }
    ;

Number: DEC_CONST     { $$ = $1; }
      | '-' DEC_CONST { $$ = -1 * $2; }
      ;


%%

int yyerror(char *msg)
{
	fprintf(stderr, "%s: stmt #%d: %s\n", the_filename, curr_statement_no, msg);
	return 0;
}

